#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <EEPROM.h>
#include <FS.h>
#include "HX711.h"
#include "index_html.h"  // In dieser Datei befindet sich die Steuerung Webseite
#include "tabelle_html.h"
#include "WEBset.h"


extern "C" {
#include "user_interface.h"
}

// #define DNS
#define LEDpin D4   // LED zum Blinken. Bei ESP-07 Pin 2. Bei ESP-01 Pin 1
const int LOADCELL_DOUT_PIN = D7;     // Waage DOUT
const int LOADCELL_SCK_PIN = D8;      // Waage SCK
String LogDatei = "Waage.txt";

float reading;
long GewichtFaktor;
boolean tariere = false;
boolean HX711_OK;

String BenuzPass = "";
char ssid[32] = "fhgdhfghd\0";
char passwort[64] = "gghdfgh\0";
char BenuzName[32] = "admin\0";
char BenuzPasswort[32] = "gghdfgh\0";
IPAddress apIP(192, 168, 168, 30);

#if defined DNS
#include <DNSServer.h>
const byte DNS_PORT = 53;
DNSServer dnsServer;
#endif

ESP8266WebServer server;
HX711 scale;
EspClass esp;
int z = 0;                   //Aktuelle EEPROM-Adresse zum lesen
#include "Setup.h"

boolean AP = 0; // Acsespoint Modus aus

void setup()                // Wird 1 Mal beim Start ausgefuehrt
{
  pinMode(LEDpin, OUTPUT);
  digitalWrite(LEDpin, 0);  // LED ON
  char inser;               // Serielle daten ablegen
  String nachricht = "";    //  Setup Formular
  EEPROM.begin(300);                                 // EEPROM initialisieren mit 200 Byts
  Serial.begin(115200);
  while (Serial.available())inser = Serial.read();  // Setup Weiche, Erkennen ob TXD/RXD gebruckt sind
  Serial.println("Start");
  delay(10);
  inser = Serial.read();
  if (inser != 'S') goto weiter;
  inser = Serial.read();
  if (inser != 't') goto weiter;
  digitalWrite(LEDpin, 0);
  Einstellen();

weiter:

//  HX711_OK = (digitalRead(LOADCELL_DOUT_PIN) == 0) ;   // Prüfen ob HX711 dran ist

scale.begin(LOADCELL_DOUT_PIN, LOADCELL_SCK_PIN);  // sonst hängt "scale.begin" in einer Indlosschleife

  // --------- WiFi Konfiguration laden -----------------------------
  z = 0;
  LeseEeprom(ssid, sizeof(ssid));        // EEPROM lesen

  if (ssid[0] == 255) // Wenn EEPROM leer ist dann:
  {
    for (int i = 0; i < 6; i++) EEPROM.write(i, '\0');
    EEPROM.commit();
    Einstellen();
  }

  LeseEeprom(passwort, sizeof(passwort));
  LeseEeprom(BenuzPasswort, sizeof(BenuzPasswort));
  Serial.println("Freie RAM = " + String(system_get_free_heap_size()));
  if (!SPIFFS.begin()) Serial.println("Failed to mount file system");

  if (ssid[0] == '\0')    //Wenn ssid leer, dann Access Point Modus starten
  {
    Serial.println("Starte in Access Point modus" );
    Serial.println("IP http://192.168.168.30");
    Serial.print("SSID: WebWaage, Passwort: ");
    Serial.println(passwort);
    WiFi.mode(WIFI_AP);      // access point modus
    WiFi.softAPConfig(apIP, apIP, IPAddress(255, 255, 255, 0));
    strncpy(ssid, "WebWaage\0", 13);
    WiFi.softAP(&ssid[0], &passwort[0]);  // Name des Wi-Fi netzes

#if defined DNS
    dnsServer.start(DNS_PORT, "*", apIP);
#endif
    AP = 1;
    digitalWrite(LEDpin, 1); // LED aus
  }
  else // Wenn ssid angegeben, dann in Stationmodus mit Router verbinden
  {
    Serial.print("Verbinde mit ");
    Serial.println(ssid);
    WiFi.mode(WIFI_STA);
    WiFi.begin(ssid, passwort);
    int    timout = 0;
    while (WiFi.status() != WL_CONNECTED)
    {
      delay(500);
      Serial.print("O");
      digitalWrite(LEDpin, !digitalRead(LEDpin));
      timout++;
      if  (timout > 20)  // Wenn Anmeldung nicht moglich
      {
        Serial.println("");
        Serial.println("Wlan verbindung fehlt");
        digitalWrite(LEDpin, 0);  // LED ON für 5 Sekunden
        delay(5000);
        digitalWrite(LEDpin, 1);
        break;
      }
    }

    if (WiFi.status() == WL_CONNECTED)
    {
      digitalWrite(LEDpin, 1);
      Serial.println("");
      Serial.println("Mit Wlan verbunden");
      Serial.print("IP Adresse: ");
      Serial.println(WiFi.localIP());
    }
  } // ENDE Stationmodus / Access Point modus Auswahl

  if (!BenuzPasswort[0])Serial.println("Serverauthentifizierung ausgeschaltet");
  else
  {
    Serial.println("Serverauthentifizierung eingeschaltet");
    Serial.print("Benutzername : ");
    Serial.println(BenuzName);
    Serial.print("Benutzerpasswort : ");
    Serial.println(BenuzPasswort);
  }

  //  Bechandlung der Ereignissen anschlissen
  server.on("/", Ereignis_Index);
  server.on("/tabelle.html", Ereignis_tabelle);
  server.on("/gewicht.php", Ereignis_gewicht);
  String mannn = "/" + LogDatei;
  server.on(mannn.c_str(), Ereignis_download);
  server.onNotFound ( Ereignis_Index );
  server.begin();  // Starte den Server
  Serial.println("HTTP Server gestartet");

  // --------------- Calibrationswerte aus EEPROM in Variable "GewichtFaktor" laden
  EEPROMAnythingRead(260, reinterpret_cast<char*>(&GewichtFaktor), sizeof(GewichtFaktor));
HX711_OK=scale.is_ready();
  
  if (HX711_OK)
  {
    scale.set_scale(GewichtFaktor);
    Serial.println(GewichtFaktor);
    scale.tare();
  }
  else Serial.println("HX711 nicht angeschlossen");
}

void Ereignis_download()  // Tabelle mit Werten hochladen
{
  if (SPIFFS.exists(LogDatei))
  {
    File file = SPIFFS.open(LogDatei, "r");
    server.streamFile(file, "application/octet-stream");
    file.close();
  }
  else  server.send(200, "text/html", "Keine Daten vorhanden");
}

boolean LogSchreiben(String Daten) // Schreibt eine Zeile in die LogDatei
{
  if (!SPIFFS.exists(LogDatei))
  {
    File DataFile = SPIFFS.open(LogDatei, "w");
    DataFile.println(Daten);
    DataFile.close();
    return true;
  }
  File DataFile = SPIFFS.open(LogDatei, "a");  // bzw erzeugt die Datei
  DataFile.println(Daten); //
  DataFile.close();
  return true;
}



void Ereignis_Index()    // Wird ausgeuehrt wenn "http://<ip address>/" aufgerufen wurde
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }
  server.sendHeader("Cache-Control", "no-cache");
  server.send(200, "text/html", indexHTML);
}

void Ereignis_tabelle()
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }
  server.sendHeader("Cache-Control", "no-cache");
  server.send(200, "text/html", tabelleHTML);
}

void Ereignis_gewicht()  //  http://<ip address>/gewicht.php
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }

  if (server.hasArg("tara"))  // http://<ip address>/gewicht.php?tara=1
  {
    if (HX711_OK)  tariere = true;   // Tarieren veranlassen
  }

  //-------- Waage calibrieren ---------------------------------

  if (server.hasArg("eichgewicht")) // http://<ip address>/gewicht.php?eichgewicht=0
  { // http://<ip address>/gewicht.php?eichgewicht=1000
    if (server.arg("eichgewicht") == "0")
    {
      scale.set_scale();
      scale.tare();
    }
    else
    {
      Serial.print("Eiche mit ");
      Serial.println(server.arg("eichgewicht"));
      GewichtFaktor = scale.get_units(10) / server.arg("eichgewicht").toFloat();
      Serial.println(GewichtFaktor);
      scale.set_scale(GewichtFaktor);
      EEPROMAnythingWrite(260, reinterpret_cast<char*>(&GewichtFaktor), sizeof(GewichtFaktor));
      EEPROM.commit();
    }
  }

  //-------- Zeile für Log Datei empfangen ---------------------------------

  if (server.arg("Artikel") != "")   // http://<ip address>/gewicht.php?Artikel=Schrauben&Gewicht=256,8+g
  {
    //String Daten = server.urlDecode(server.arg("Artikel")) + ";" + server.arg("Gewicht");
    String Daten = server.arg("Artikel") + "\t";
    String GewichtMitEinheit = server.arg("Gewicht");
    GewichtMitEinheit.replace(" ", "\t");
    Daten = Daten + GewichtMitEinheit;
    LogSchreiben(Daten);
  }
  //-------- Log Datei löschen ---------------------------------

  if (server.arg("tabelle") == "loeschen")
  {
    if (SPIFFS.exists(LogDatei)) SPIFFS.remove(LogDatei);
    Serial.println("Tabelle Loeschen");
  }


  String Antwort = String(reading) ;
  if (!HX711_OK) Antwort = "HX711";
  server.sendHeader("Cache-Control", "no-cache, no-store, must-revalidate");
  server.sendHeader("pragma", "no-cache");
  server.sendHeader("Expires", "-1");
  server.send(200, "text/plain", Antwort); // Antwort an Internet Browser senden
}


void loop()
{
  if (tariere)
  {
    server.handleClient();
    scale.tare();
    Serial.println("Tariere");
    tariere = false;
  }

  if (HX711_OK)
  {
    if (scale.is_ready()) reading = scale.get_units(1);
  }

  server.handleClient();
#if defined DNS
  dnsServer.processNextRequest();
#endif
}



// Write any data structure or variable to EEPROM
int EEPROMAnythingWrite(int pos, char *zeichen, int lenge)
{
  for (int i = 0; i < lenge; i++)
  {
    EEPROM.write(pos + i, *zeichen);
    zeichen++;
  }
  return pos + lenge;
}

// Read any data structure or variable from EEPROM
int EEPROMAnythingRead(int pos, char *zeichen, int lenge)
{
  for (int i = 0; i < lenge; i++)
  {
    *zeichen = EEPROM.read(pos + i);
    zeichen++;
  }
  return pos + lenge;
}
